@@ -23,40 +23,14 @@
 ;;; shell scripts, batch scripts and environment variables.
 ;;; jfa 02/07/04
 
-(defvar *maxima-prefix*)
 (defvar *maxima-topdir*)        ;; top-level installation or build directory
 (defvar *maxima-imagesdir*)
 (defvar *maxima-sharedir*)
 (defvar *maxima-srcdir*)
 (defvar *maxima-docdir*)
-(defvar *maxima-infodir*)
-(defvar *maxima-htmldir*)
 (defvar *maxima-layout-autotools*)
-(defvar *maxima-userdir*)
-(defvar *maxima-initmac* "maxima-init.mac")
-(defvar *maxima-initlisp* "maxima-init.lisp")
-(defvar *maxima-tempdir*)
-(defvar *maxima-lang-subdir* nil)
 (defvar *maxima-demodir*)
 (defvar *maxima-objdir*)		;; Where to store object (fasl) files.
-(defvar $maxima_frontend nil "The frontend maxima is used with.")
-(defvar $maxima_frontend_version nil "The version of the maxima frontend.")
-
-(eval-when (:load-toplevel :compile-toplevel :execute)
-  (defmacro def-lisp-shadow (root-name)
-    "Create a maxima variable $root_name that is an alias for the lisp name *root-name*.
-When one changes, the other does too."
-    (let ((maxima-name (intern (concatenate 'string "$"
-					    (substitute #\_ #\- (string root-name)))))
-	  (lisp-name (intern (concatenate 'string "*" (string root-name) "*"))))
-      `(progn
-	 (defmvar ,maxima-name)
-	 (putprop ',maxima-name 'shadow-string-assignment 'assign)
-	 (putprop ',maxima-name ',lisp-name 'lisp-shadow)))))
-
-(def-lisp-shadow maxima-tempdir)
-(def-lisp-shadow maxima-userdir)
-(def-lisp-shadow maxima-objdir)
 
 (defun shadow-string-assignment (var value)
   (cond
@@ -97,20 +71,6 @@ When one changes, the other does too."
 	#+ecl "ecl"
 	#-(or clisp cmu scl sbcl gcl allegro openmcl abcl lispworks ecl) "unknownlisp")
 
-(defvar $file_search_lisp nil
-  "Directories to search for Lisp source code.")
-
-(defvar $file_search_maxima nil
-  "Directories to search for Maxima source code.")
-
-(defvar $file_search_demo nil
-  "Directories to search for demos.")
-
-(defvar $file_search_usage nil)
-
-(defvar $file_search_tests nil
-  "Directories to search for maxima test suite")
-
 (defun maxima-parse-dirstring (str)
   (let ((sep "/"))
     (if (position (character "\\") str)
@@ -263,7 +223,8 @@ When one changes, the other does too."
 	(maxima-userdir-env (maxima-getenv "MAXIMA_USERDIR"))
 	(maxima-docprefix-env (maxima-getenv "MAXIMA_DOC_PREFIX"))
 	(maxima-tempdir-env (maxima-getenv "MAXIMA_TEMPDIR"))
-	(maxima-objdir-env (maxima-getenv "MAXIMA_OBJDIR")))
+	(maxima-objdir-env (maxima-getenv "MAXIMA_OBJDIR"))
+	(maxima-htmldir-env (maxima-getenv "MAXIMA_HTMLDIR")))
     ;; MAXIMA_DIRECTORY is a deprecated substitute for MAXIMA_PREFIX
     (unless maxima-prefix-env
       (setq maxima-prefix-env (maxima-getenv "MAXIMA_DIRECTORY")))
@@ -295,20 +256,13 @@ When one changes, the other does too."
                          (concatenate 'string *maxima-userdir* "/binary"))
                        "/" (maxima-version1) "/" *maxima-lispname* "/" (lisp-implementation-version1)))
 
+    (when maxima-htmldir-env
+      (setq *maxima-htmldir* (combine-path (maxima-parse-dirstring maxima-htmldir-env) "doc" "info")))
+
     ;; On ECL the testbench fails mysteriously if this directory doesn't exist =>
     ;; let's create it by hand as a workaround.
     #+ecl (ensure-directories-exist (concatenate 'string *maxima-objdir* "/"))
     
-    ; On Windows Vista gcc requires explicit include
-    #+gcl
-    (when (string= *autoconf-windows* "true")
-      (let ((mingw-gccver (maxima-getenv "mingw_gccver")))
-	(when mingw-gccver
-	  (setq compiler::*cc*
-		(concatenate 'string compiler::*cc* " -I\"" *maxima-prefix* "\\include\""
-			     " -I\"" *maxima-prefix* "\\lib\\gcc-lib\\mingw32\\"
-			     mingw-gccver "\\include\" ")))))
-
     ; Assign initial values for Maxima shadow variables
     (setq $maxima_userdir *maxima-userdir*)
     (setf (gethash '$maxima_userdir *variable-initial-values*) *maxima-userdir*)
@@ -379,7 +333,7 @@ When one changes, the other does too."
                           ns
                           (if (eql #\/ (char ns (1- (length ns)))) "" "/")
                           "*"
-                          #+(or :clisp :sbcl :ecl :openmcl) "/")
+                          #+(or :clisp :sbcl :ecl :openmcl :gcl) "/")
              #+openmcl :directories #+openmcl t))
 
 (defun unix-like-basename (path)
@@ -431,6 +385,11 @@ When one changes, the other does too."
     ;; option to have an equivalent long option.  No check is made for
     ;; this, so please make sure this holds.  Or change the code in
     ;; process-args in command-line.lisp.
+    ;;
+    ;; The help strings should not have any special manual formatting
+    ;; but extraneous white space is ok.  They are automatically
+    ;; printed with extraneous whitespace (including newlines) removed
+    ;; and lines wrapped neatly.
     (setf maxima-options
 	  (list
 	   (make-cl-option :names '("-b" "--batch")
@@ -488,20 +447,46 @@ When one changes, the other does too."
 			   :help-string "Use  <directory> for user directory (default is %USERPROFILE%/maxima for Windows, and $HOME/.maxima for other operating systems).")
  	   (make-cl-option :names '("--init")
 			   :argument "<file>"
-			   :action #'(lambda (file)
-				       (setf *maxima-initmac* (concatenate 'string file ".mac"))
-				       (setf *maxima-initlisp* (concatenate 'string file ".lisp")))
-			   :help-string (format nil "Set the name of the Maxima & Lisp initialization files to <file>.mac & <file>.lisp (default is ~a)" (subseq *maxima-initmac* 0 (- (length *maxima-initmac*) 4))))
- 	   (make-cl-option :names '("--init-mac")
+			   :action
+			   #'(lambda (file)
+			       (flet
+				   ((get-base-name (f)
+				      ;; Strip off everything before
+				      ;; the last "/" (or "\").  Then
+				      ;; strip off everything after
+				      ;; the last dot.
+				      (let* ((dot (position #\. f :from-end t))
+					     (dir (position-if
+						   #'(lambda (c)
+						       (member c '(#\/ #\\)))
+						   f
+						   :from-end t))
+					     (base (subseq f (if dir (1+ dir) 0) dot)))
+					(when (or dot dir)
+					  (mtell (intl:gettext "Warning: Using basename ~S for init files instead of ~S" )
+						 base f))
+					base)))
+				 (let ((base-name (get-base-name file)))
+				   (setf *maxima-initmac*
+					 (concatenate 'string base-name ".mac"))
+				   (setf *maxima-initlisp*
+					 (concatenate 'string base-name ".lisp")))))
+			   :help-string (format nil "Set the base name of the Maxima & Lisp initialization files (default is ~s.)  The last extension and any directory parts are removed to form the base name.  The resulting files, <base>.mac and <base>.lisp are only searched for in userdir (see --userdir option).  This may be specified for than once, but only the last is used."
+						(subseq *maxima-initmac* 0
+							(- (length *maxima-initmac*) 4))))
+ 	   #+nil
+	   (make-cl-option :names '("--init-mac")
 			   :argument "<file>"
 			   :action #'(lambda (file)
 				       (setf *maxima-initmac* file))
-			   :help-string (format nil "Set the name of the Maxima initialization file (default is ~a)" *maxima-initmac*))
- 	   (make-cl-option :names '("--init-lisp")
+			   :help-string (format nil "Set the name of the Maxima initialization file (default is ~s)"
+						*default-maxima-initmac*))
+ 	   #+nil
+	   (make-cl-option :names '("--init-lisp")
 			   :argument "<file>"
 			   :action #'(lambda (file)
 				       (setf *maxima-initlisp* file))
-			   :help-string (format nil "Set the name of the Lisp initialization file (default is ~a)" *maxima-initlisp*))
+			   :help-string (format nil "Set the name of the Lisp initialization file (default is ~s)" *default-maxima-initlisp*))
 	   (make-cl-option :names '("-l" "--lisp")
 			   :argument "<lisp>"
 			   :action nil
@@ -510,13 +495,29 @@ When one changes, the other does too."
 			   :action 'list-avail-action
 			   :help-string
 			   "List the installed version/lisp combinations.")
-	   (make-cl-option :names '("-p" "--preload-lisp")
-			   :argument "<lisp-file>"
+	   ;; --preload-lisp is left for backward compatibility.  We
+	   ;; no longer distinguish between mac and lisp files.  Any
+	   ;; file type that $LOAD supports is acceptable.
+	   ;; "--init-mac" and "--init-lisp" are now also (deprecated)
+	   ;; aliases for --preload.
+	   (make-cl-option :names '("-p" "--preload" "--preload-lisp" "--init-mac" "--init-lisp")
+			   :argument "<file>"
 			   :action #'(lambda (file)
-				       #-sbcl (load file) #+sbcl (with-compilation-unit nil (load file)))
-			   :help-string "Preload <lisp-file>.")
+				       ;; $loadprint T so we can see the file being loaded.
+				       (let (($loadprint t))
+				       ($load file)))
+			   :help-string
+        "Preload <file>, which may be any file time accepted by
+        Maxima's LOAD function.  The <file> is loaded before any other
+        system initialization is done.  This will be searched for in
+        the locations given by file_search_maxima and
+        file_search_lisp.  This can be specified multiple times to
+        load multiple files. The equivalent options --preload-lisp,
+        --init-mac, and --init-lisp are deprecated.")
 	   (make-cl-option :names '("-q" "--quiet")
-			   :action #'(lambda () (declare (special *maxima-quiet*)) (setq *maxima-quiet* t))
+			   :action #'(lambda ()
+				       (declare (special *maxima-quiet*))
+				       (setq *maxima-quiet* t))
 			   :help-string "Suppress Maxima start-up message.")
 	   (make-cl-option :names '("-r" "--run-string")
 			   :argument "<string>"
@@ -551,21 +552,25 @@ When one changes, the other does too."
 			   :help-string
 			   "Display the default installed version.")
 	   (make-cl-option :names '("--very-quiet")
-			   :action #'(lambda () (declare (special *maxima-quiet* *display-labels-p*))
-					     (setq *maxima-quiet* t *display-labels-p* nil))
+			   :action #'(lambda ()
+				       (declare (special *maxima-quiet*))
+				       (setq *maxima-quiet* t *display-labels-p* nil))
 			   :help-string "Suppress expression labels and Maxima start-up message.")
 	   (make-cl-option :names '("-X" "--lisp-options")
 			   :argument "<Lisp options>"
-			   :action #'(lambda (&rest opts) (declare (special *maxima-quiet*))
-				       (unless *maxima-quiet* (format t "Lisp options: ~A" opts)))
+			   :action #'(lambda (&rest opts)
+				       (declare (special *maxima-quiet*))
+				       (unless *maxima-quiet*
+					 (format t "Lisp options: ~A" opts)))
 			   :help-string "Options to be given to the underlying Lisp")
+	   (make-cl-option :names '("--no-init" "--norc")
+			   :action #'(lambda ()
+				       (setf *maxima-load-init-files* nil))
+			   :help-string "Do not load the init file(s) on startup")
 			   ))
     (process-args (get-application-args) maxima-options))
   (values input-stream batch-flag))
 
-;; A list of temporary files that can be deleted on leaving maxima
-(defvar *temp-files-list* (make-hash-table :test 'equal))
-
 ;; Delete all files *temp-files-list* contains.
 (defun delete-temp-files ()
   (maphash #'(lambda(filename param)
@@ -591,35 +596,31 @@ When one changes, the other does too."
 	(catch 'to-lisp
 	  (setf (values input-stream batch-flag)
 		(process-maxima-args input-stream batch-flag))
+	  (load-user-init-file)
 	  (loop
 	   (with-simple-restart (macsyma-quit "Maxima top-level")
 				(macsyma-top-level input-stream batch-flag))))
       (delete-temp-files)
     )))
 
-(defun disable-some-lisp-warnings ()
-  ;; Suppress warnings about redefining functions;
-  ;; it appears that only Clisp and SBCL emit these warnings
-  ;; (ECL, GCL, CMUCL, and Clozure CL apparently do not).
-  ;; Such warnings are generated by the autoload mechanism.
-  ;; I guess it is plausible that we could also avoid the warnings by
-  ;; reworking autoload to not trigger them. I don't have enough
-  ;; motivation to attempt that right now.
-  #+sbcl (setq sb-ext:*muffled-warnings* '(or sb-kernel:redefinition-with-defun sb-kernel:uninteresting-redefinition))
-  #+sbcl (declaim (sb-ext:muffle-conditions sb-ext:compiler-note))
-  #+clisp (setq custom:*suppress-check-redefinition* t)
+;; If the user specified an init file, use it.  If not, use the
+;; default init file in the userdir directory, but only if it
+;; exists.  A user-specified init file is searched in the search
+;; paths.
 
-  ;; Suppress compiler output messages.
-  ;; These include the "0 errors, 0 warnings" message output from Clisp,
-  ;; and maybe other messages from other Lisps.
-  (setq *compile-verbose* nil))
-
-(defun enable-some-lisp-warnings ()
-  ;; SB-KERNEL:UNINTERESTING-REDEFINITION appears to be the default value.
-  #+sbcl (setq sb-ext:*muffled-warnings* 'sb-kernel:uninteresting-redefinition)
-  #+sbcl (declaim (sb-ext:unmuffle-conditions sb-ext:compiler-note))
-  #+clisp (setq custom:*suppress-check-redefinition* nil)
-  (setq *compile-verbose* t))
+(defun load-user-init-file ()
+    (flet
+	((maybe-load-init-file (loader default-init)
+	   (let ((init-file
+		   (combine-path *maxima-userdir* default-init)))
+	     (when (and *maxima-load-init-files*
+			(file-exists-p init-file))
+	       (format t "Loading ~A~%" init-file)
+	       (funcall loader init-file)))))
+      ;; Catch errors from $load or $batchload which can throw to 'macsyma-quit.
+      (catch 'macsyma-quit
+	(maybe-load-init-file #'$load *maxima-initlisp*)
+	(maybe-load-init-file #'$batchload *maxima-initmac*))))
 
 (defun initialize-runtime-globals ()
   (setf *load-verbose* nil)
@@ -655,6 +656,17 @@ When one changes, the other does too."
 
   #+sbcl (setf *read-default-float-format* 'double-float)
 
+  ;; GCL: disable readline symbol completion,
+  ;; leaving other functionality (line editing, anything else?) enabled.
+  ;;
+  ;; This is kind of terrible. I don't see a flag to only disable completion,
+  ;; or a way to set the symbol list to Maxima symbols and disable case inversion,
+  ;; so set the completion prefix to a nonexistent package.
+  ;; If ever package BLURFLE is actually defined, and contains external symbols,
+  ;; those symbols will be completed. I can live with that.
+  
+  #+gcl (setq si::*readline-prefix* "BLURFLE:")
+
   (initialize-real-and-run-time)
   (intl::setlocale)
   (set-locale-subdir)
@@ -667,7 +679,38 @@ When one changes, the other does too."
                                  (if *maxima-layout-autotools*
                                      "/share/locale/"
                                      "/locale/")))
-          intl::*locale-directories*)))
+          intl::*locale-directories*))
+  ;; Set up $browser for displaying help in browser.
+  (cond ((and (boundp '*autoconf-windows*)
+	      (string-equal *autoconf-windows* "true"))
+	 ;; Starts the default browser on Windows.
+	 (setf $browser "start ~A"))
+	((boundp '*autoconf-host*)
+	 ;; Determine what kind of OS we're using from the host and
+	 ;; set up the default browser appropriately.
+	 (cond ((pregexp:pregexp-match-positions "(?:darwin)" *autoconf-host*)
+		(setf $browser "open '~A'"))
+	       ((pregexp:pregexp-match-positions "(?i:linux)" *autoconf-host*)
+		(setf $browser "xdg-open '~A'")))))
+  (setf %e-val (mget '$%e '$numer))
+
+  ;; Initialize *bigprimes* here instead of globals.lisp because we
+  ;; need the NEXT-PRIME function.
+  (setf *bigprimes*
+	(loop with p = (ash most-positive-fixnum -1)
+	      repeat 20
+	      do (setq p (next-prime (1- p) -1))
+	      collect p))
+  ;; Initialize *alpha and $pointbound.  Since both of these are
+  ;; defmvars, we need to set the initial values appropriately too so
+  ;; they get reset correctly.
+  (setf *alpha (car *bigprimes*))
+  (setf (gethash '*alpha *variable-initial-values*)
+	(car *bigprimes*))
+  (setf $pointbound *alpha)
+  (setf (gethash '$pointbound *variable-initial-values*)
+	*alpha)
+  (values))
 
 (defun adjust-character-encoding ()
   #+sbcl (setf sb-impl::*default-external-format* :utf-8)
@@ -681,6 +724,7 @@ When one changes, the other does too."
     ;; utf-8.  The handler-bind is needed in case the filename
     ;; encoding was already set to something else; we forcibly change
     ;; it to utf-8. (Is that right?)
+    (setf stream:*default-external-format* :utf-8)
     (stream:set-system-external-format :utf-8 :utf-8))
   #+clisp
   (ignore-errors
@@ -793,9 +837,6 @@ When one changes, the other does too."
 
 (defun maxima-load-pathname-directory ()
   "Return the directory part of *load-pathname*."
-  (let ((path #-gcl *load-pathname*
-              ;; Accommodate standard and nonstandard definitions of *LOAD-PATHNAME* in GCL.
-              ;; This can go away someday when nonstandard GCL's (<= 2.6.12) are ancient history.
-              #+gcl (symbol-value (or (find-symbol "*LOAD-PATHNAME*" :sys) (find-symbol "*LOAD-PATHNAME*" :common-lisp)))))
+  (let ((path *load-pathname*))
     (make-pathname :directory (pathname-directory path)
                    :device (pathname-device path))))
